/*
 * Created on 11.11.2004
 *
 * SVN header information:
 *  $Author: LBST-PF-3\orahn $
 *  $Rev: 2446 $
 *  $Date: 2006-09-12 14:57:25 +0200 (Di, 12 Sep 2006) $
 *  $Id: PirolCsvReaderDialog.java 2446 2006-09-12 12:57:25Z LBST-PF-3\orahn $s
 */
package de.fhOsnabrueck.jump.pirol.datasources.pirolCSV;

import java.awt.Component;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.HeadlessException;
import java.awt.Insets;
import java.awt.LayoutManager;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;

import de.fhOsnabrueck.jump.pirol.ui.tools.DialogTools;
import de.fhOsnabrueck.jump.pirol.utilities.i18n.PirolPlugInMessages;

/**
 * @author Ole Rahn
 * <br>
 * <br>FH Osnabr&uuml;ck - University of Applied Sciences Osnabr&uuml;ck,
 * <br>Project: PIROL (2005),
 * <br>Subproject: Daten- und Wissensmanagement
 * 
 * @version $Rev: 2446 $
 */
public class PirolCsvReaderDialog extends JDialog implements ActionListener {

    private static final long serialVersionUID = 8214818341892396934L;

    private String[] lines = null;
	private String[] validSeparatorsString;
	private String separator = "\t";
	private int xcol=0, ycol=0, zcol=-1;
	
	private JComboBox nordBox, ostBox, trennerBox;
	private JButton okButton, cancelButton;
	private JPanel gui;
	
	private boolean wasInputOK = false;
	
	
	public void updateComboBoxes(){
		nordBox.removeAllItems();
		ostBox.removeAllItems();
		
		String[] fields = this.getFieldsUsingSeparator();
		
		for ( int i=0; i<fields.length; i++){
			nordBox.addItem(fields[i]);
			ostBox.addItem(fields[i]);
		}
		
		
	}
	
	public PirolCsvReaderDialog(Frame owner, String[] lines) throws HeadlessException {
		super(owner, PirolPlugInMessages.getString("csv-dialog-title"), true);
		
		this.setEnabled(true);
		
		this.gui = new JPanel();
		
		this.lines = lines;

		validSeparatorsString = new String[PirolCSV_Parser.validSepatators.length];
		
		for (int i=0; i<PirolCSV_Parser.validSepatators.length; i++){
			switch (PirolCSV_Parser.validSepatators[i].charAt(0)){
				case '\t':
					validSeparatorsString[i] = PirolPlugInMessages.getString("tab");
					break;
				case ' ':
					validSeparatorsString[i] = PirolPlugInMessages.getString("space");
					break;
				case ',':
					validSeparatorsString[i] = PirolPlugInMessages.getString("comma");
					break;
				case ';':
					validSeparatorsString[i] = PirolPlugInMessages.getString("semikolon");
					break;
				default:
					validSeparatorsString[i] = PirolCSV_Parser.validSepatators[i];
					break;
			}
		}

		this.setupDialog();
		DialogTools.centerOnScreen(this);
	}
	
	public String getSeparator() {
		return separator;
	}

	public int getXcol() {
		return xcol;
	}

	public int getYcol() {
		return ycol;
	}

	public int getZcol() {
		return zcol;
	}
	
	


	public boolean isInputOK() {
		return wasInputOK;
	}

	public void setSeparator(int separator) {
		if (separator<PirolCSV_Parser.validSepatators.length || separator < 0)
			this.separator = PirolCSV_Parser.validSepatators[separator];
		else
			this.separator = null;
	}
	/**
	 * @param xcol The xcol to set.
	 */
	public void setXcol(int xcol) {
		//System.out.println("dialog.setXcol");
		this.xcol = xcol;
	}
	/**
	 * @param ycol The ycol to set.
	 */
	public void setYcol(int ycol) {
		//System.out.println("dialog.setYcol");
		this.ycol = ycol;
	}
	/**
	 * @param zcol The zcol to set.
	 */
	public void setZcol(int zcol) {
		this.zcol = zcol;
	}
	public Component add(Component c){
		return this.gui.add(c);
	}
	public void setLayoutManager(LayoutManager l){
		if (l!=null && this.gui != null)
			this.gui.setLayout(l);
	}
	
	public void makeLayout(){
		this.gui.doLayout();
		this.getContentPane().add(this.gui);
		this.pack();
	}

	private String[] getFieldsUsingSeparator(){
		return lines[0].split(this.separator);
	}
	
	private void setupDialog() {
		GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        
        c.insets = new Insets(5,5,5,5);

        if ( this.gui != null ){
        	this.setLayoutManager(gridbag);
        }else {
            /*
        	System.out.println("mist");
        	System.out.flush();
            */
        }

        c.fill = GridBagConstraints.BOTH;
        c.weightx = 1.0;
        
        JLabel nordLabel = new JLabel(PirolPlugInMessages.getString("n-s-coordinate"));
        gridbag.setConstraints(nordLabel, c);
        this.add(nordLabel);
        
        nordBox = new ReactiveCSVComboBox( getFieldsUsingSeparator(), ReactiveCSVComboBox.Y, this );
        nordBox.addActionListener(nordBox);
        c.gridwidth = GridBagConstraints.REMAINDER; //end row
        gridbag.setConstraints(nordBox, c);
        this.add(nordBox);
        
        c.weightx = 0.0;                  //reset to the default
        c.gridwidth = 1;                //reset to the default
        c.gridheight = 2;
        JLabel ostLabel = new JLabel(PirolPlugInMessages.getString("e-w-coordinate"));
        gridbag.setConstraints(ostLabel, c);
        this.add(ostLabel);
        
        ostBox = new ReactiveCSVComboBox( getFieldsUsingSeparator(), ReactiveCSVComboBox.X, this );
        ostBox.addActionListener(ostBox);
        c.gridwidth = GridBagConstraints.REMAINDER; //end row
        gridbag.setConstraints(ostBox, c);
        this.add(ostBox);
        
        c.weightx = 0.0;                  //reset to the default
        c.gridwidth = 1;                //reset to the default
        c.gridheight = 3;
        JLabel trennerLabel = new JLabel(PirolPlugInMessages.getString("separator"));
        gridbag.setConstraints(trennerLabel, c);
        this.add(trennerLabel);
        
        trennerBox = new ReactiveCSVComboBox( this.validSeparatorsString, ReactiveCSVComboBox.SEPARATOR, this );
        trennerBox.addActionListener(trennerBox);
        c.gridwidth = GridBagConstraints.REMAINDER; //end row
        gridbag.setConstraints(trennerBox, c);
        this.add(trennerBox);
        
        c.weightx = 0.0;                  //reset to the default
        c.gridwidth = 1;                //reset to the default
        c.gridheight = 4;
        this.okButton = new JButton(PirolPlugInMessages.getString("ok"));
        this.okButton.setActionCommand(this.okButton.getText());
		this.okButton.addActionListener(this);
		gridbag.setConstraints(this.okButton, c);
		this.add(this.okButton);
		
		this.cancelButton = new JButton(PirolPlugInMessages.getString("cancel"));
		c.gridwidth = GridBagConstraints.REMAINDER;;
		this.cancelButton.setActionCommand(this.cancelButton.getText());
		this.cancelButton.addActionListener(this);
		gridbag.setConstraints(this.cancelButton, c);
		this.add(this.cancelButton);
        
        this.makeLayout();
        
	}


	public void actionPerformed(ActionEvent arg0) {
		JButton button = (JButton)arg0.getSource();
		
		if (button.getActionCommand().equals(this.okButton.getActionCommand())){
			this.wasInputOK = true;
		} else {
			this.wasInputOK = false;
			this.xcol = -1;
			this.ycol = -1;
			this.separator = null;
		}
		
		this.setVisible(false);
	}
	
}
